var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/Observable';
import { Headers, RequestMethod, Response, ResponseOptions } from '@angular/http';
import { HTTP } from './cordova-http-plugin';
var NativeHttpConnection = (function () {
    function NativeHttpConnection(req, nativeHttp, baseResponseOptions) {
        var _this = this;
        var allowedRequestMethods = [
            RequestMethod.Get,
            RequestMethod.Post,
            RequestMethod.Put,
            RequestMethod.Delete
        ];
        if (allowedRequestMethods.indexOf(req.method) === -1) {
            throw 'Only GET, POST, PUT and DELETE methods are supported by the current Native HTTP version';
        }
        this.request = req;
        this.response = new Observable(function (responseObserver) {
            var headers = req.headers.toJSON();
            Object.keys(headers).map(function (key) {
                if (headers[key].length > 1) {
                    throw "Header " + key + " contains more than one value";
                }
                headers[key] = headers[key][0];
            });
            var body;
            // 1 stands for ContentType.JSON. Stupid Angular doesn't export ContentType
            if (req.detectContentTypeFromBody() === 1) {
                body = req.json();
            }
            else {
                body = _this.getBodyParams(req.getBody());
            }
            var requestMethod = _this.detectRequestMethod(req);
            nativeHttp[requestMethod](encodeURI(req.url), body, headers).then(function (response) {
                _this.fireResponse(responseObserver, new ResponseOptions({
                    body: response.data,
                    status: response.status,
                    headers: new Headers(response.headers)
                }), baseResponseOptions);
            }).catch(function (error) {
                _this.fireResponse(responseObserver, new ResponseOptions({
                    body: error.error,
                    status: error.status || 599,
                    headers: new Headers(error.headers)
                }), baseResponseOptions);
            });
        });
    }
    NativeHttpConnection.prototype.detectRequestMethod = function (req) {
        var requestMethod;
        switch (req.method) {
            case RequestMethod.Post:
                requestMethod = 'post';
                break;
            case RequestMethod.Put:
                requestMethod = 'put';
                break;
            case RequestMethod.Delete:
                requestMethod = 'delete';
                break;
            default:
                requestMethod = 'get';
        }
        // 1 stands for ContentType.JSON. Stupid Angular doesn't export ContentType
        if (req.detectContentTypeFromBody() === 1 && requestMethod === 'post') {
            requestMethod = 'postJson';
        }
        return requestMethod;
    };
    NativeHttpConnection.prototype.fireResponse = function (responseObserver, responseOptions, baseResponseOptions) {
        if (baseResponseOptions) {
            responseOptions = baseResponseOptions.merge(responseOptions);
        }
        var response = new Response(responseOptions);
        response.ok = (response.status >= 200 && response.status < 300);
        if (response.ok) {
            responseObserver.next(response);
            responseObserver.complete();
        }
        else {
            responseObserver.error(response);
        }
    };
    NativeHttpConnection.prototype.getBodyParams = function (query) {
        if (!query) {
            return {};
        }
        return (/^[?#]/.test(query) ? query.slice(1) : query)
            .split('&')
            .reduce(function (params, param) {
            var _a = param.split('='), key = _a[0], value = _a[1];
            params[key] = value ? decodeURIComponent(value.replace(/\+/g, ' ')) : '';
            return params;
        }, {});
    };
    return NativeHttpConnection;
}());
export { NativeHttpConnection };
var NativeHttpBackend = (function () {
    function NativeHttpBackend(nativeHttp, baseResponseOptions) {
        this.nativeHttp = nativeHttp;
        this.baseResponseOptions = baseResponseOptions;
    }
    NativeHttpBackend.prototype.createConnection = function (request) {
        return new NativeHttpConnection(request, this.nativeHttp, this.baseResponseOptions);
    };
    NativeHttpBackend = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [HTTP,
            ResponseOptions])
    ], NativeHttpBackend);
    return NativeHttpBackend;
}());
export { NativeHttpBackend };
//# sourceMappingURL=native-http-backend.js.map